@@ -1,14 +1,23 @@
 package filestore
 
 import (
+	"strings"
 	"time"
 
 	"gitlab.com/gitlab-org/gitlab-workhorse/internal/api"
+	"gitlab.com/gitlab-org/gitlab-workhorse/internal/config"
 )
 
 // DefaultObjectStoreTimeout is the timeout for ObjectStore upload operation
 const DefaultObjectStoreTimeout = 4 * time.Hour
 
+type ObjectStorageConfig struct {
+	Provider string
+
+	S3Credentials config.S3Credentials
+	S3Config      config.S3Config
+}
+
 // SaveFileOpts represents all the options available for saving a file to object store
 type SaveFileOpts struct {
 	// TempFilePrefix is the prefix used to create temporary local file
@@ -25,7 +34,12 @@ type SaveFileOpts struct {
 	PresignedDelete string
 	// HTTP headers to be sent along with PUT request
 	PutHeaders map[string]string
-
+	// Whether to ignore Rails pre-signed URLs and have Workhorse directly access object storage provider
+	UseWorkhorseClient bool
+	// If UseWorkhorseClient is true, this is the temporary object name to store the file
+	RemoteTempObjectID string
+	// Workhorse object storage client (e.g. S3) parameters
+	ObjectStorageConfig ObjectStorageConfig
 	// Deadline it the S3 operation deadline, the upload will be aborted if not completed in time
 	Deadline time.Time
 
@@ -40,6 +54,11 @@ type SaveFileOpts struct {
 	PresignedAbortMultipart string
 }
 
+// UseWorkhorseClientEnabled checks if the options require direct access to object storage
+func (s *SaveFileOpts) UseWorkhorseClientEnabled() bool {
+	return s.UseWorkhorseClient && s.ObjectStorageConfig.IsValid() && s.RemoteTempObjectID != ""
+}
+
 // IsLocal checks if the options require the writing of the file on disk
 func (s *SaveFileOpts) IsLocal() bool {
 	return s.LocalTempPath != ""
@@ -63,13 +82,21 @@ func GetOpts(apiResponse *api.Response) *SaveFileOpts {
 	}
 
 	opts := SaveFileOpts{
-		LocalTempPath:   apiResponse.TempPath,
-		RemoteID:        apiResponse.RemoteObject.ID,
-		RemoteURL:       apiResponse.RemoteObject.GetURL,
-		PresignedPut:    apiResponse.RemoteObject.StoreURL,
-		PresignedDelete: apiResponse.RemoteObject.DeleteURL,
-		PutHeaders:      apiResponse.RemoteObject.PutHeaders,
-		Deadline:        time.Now().Add(timeout),
+		LocalTempPath:      apiResponse.TempPath,
+		RemoteID:           apiResponse.RemoteObject.ID,
+		RemoteURL:          apiResponse.RemoteObject.GetURL,
+		PresignedPut:       apiResponse.RemoteObject.StoreURL,
+		PresignedDelete:    apiResponse.RemoteObject.DeleteURL,
+		PutHeaders:         apiResponse.RemoteObject.PutHeaders,
+		UseWorkhorseClient: apiResponse.RemoteObject.UseWorkhorseClient,
+		RemoteTempObjectID: apiResponse.RemoteObject.RemoteTempObjectID,
+		Deadline:           time.Now().Add(timeout),
+	}
+
+	objectStorageParams := apiResponse.RemoteObject.ObjectStorage
+	if opts.UseWorkhorseClient && objectStorageParams != nil {
+		opts.ObjectStorageConfig.Provider = objectStorageParams.Provider
+		opts.ObjectStorageConfig.S3Config = objectStorageParams.S3Config
 	}
 
 	// Backwards compatibility to ensure API servers that do not include the
@@ -88,3 +115,24 @@ func GetOpts(apiResponse *api.Response) *SaveFileOpts {
 
 	return &opts
 }
+
+func (c *ObjectStorageConfig) IsAWS() bool {
+	return strings.EqualFold(c.Provider, "AWS") || strings.EqualFold(c.Provider, "S3")
+}
+
+func (c *ObjectStorageConfig) IsValid() bool {
+	return c.S3Config.Bucket != "" && c.S3Config.Region != "" && c.credentialsValid()
+}
+
+func (c *ObjectStorageConfig) credentialsValid() bool {
+	// We need to be able to distinguish between two cases of AWS access:
+	// 1. AWS access via key and secret, but credentials not configured in Workhorse
+	// 2. IAM instance profiles used
+	if c.S3Config.UseIamProfile {
+		return true
+	} else if c.S3Credentials.AwsAccessKeyID != "" && c.S3Credentials.AwsSecretAccessKey != "" {
+		return true
+	}
+
+	return false
+}
